@@ -30,6 +30,7 @@
 #include "wayland/meta-wayland.h"
 #include "window-private.h"
 #include "tests/test-utils.h"
+#include "x11/meta-x11-display-private.h"
 
 typedef struct {
   GHashTable *clients;
@@ -40,7 +41,7 @@ typedef struct {
 } TestCase;
 
 static gboolean
-test_case_alarm_filter (MetaDisplay           *display,
+test_case_alarm_filter (MetaX11Display        *x11_display,
                         XSyncAlarmNotifyEvent *event,
                         gpointer               data)
 {
@@ -48,12 +49,12 @@ test_case_alarm_filter (MetaDisplay           *display,
   GHashTableIter iter;
   gpointer key, value;
 
-  if (async_waiter_alarm_filter (test->waiter, display, event))
+  if (async_waiter_alarm_filter (test->waiter, x11_display, event))
     return TRUE;
 
   g_hash_table_iter_init (&iter, test->clients);
   while (g_hash_table_iter_next (&iter, &key, &value))
-    if (test_client_alarm_filter (value, display, event))
+    if (test_client_alarm_filter (value, x11_display, event))
       return TRUE;
 
   return FALSE;
@@ -102,8 +103,8 @@ test_case_new (void)
                                             test_case_log_func,
                                             test);
 
-  meta_display_set_alarm_filter (meta_get_display (),
-                                 test_case_alarm_filter, test);
+  meta_x11_display_set_alarm_filter (meta_get_display ()->x11_display,
+                                     test_case_alarm_filter, test);
 
   test->clients = g_hash_table_new (g_str_hash, g_str_equal);
   test->waiter = async_waiter_new ();
@@ -212,7 +213,7 @@ test_case_assert_stacking (TestCase *test,
   GString *expected_string = g_string_new (NULL);
   int i;
 
-  meta_stack_tracker_get_stack (display->screen->stack_tracker, &windows, &n_windows);
+  meta_stack_tracker_get_stack (display->stack_tracker, &windows, &n_windows);
   for (i = 0; i < n_windows; i++)
     {
       MetaWindow *window = meta_display_lookup_stack_id (display, windows[i]);
@@ -222,7 +223,7 @@ test_case_assert_stacking (TestCase *test,
            * is managed as a MetaWindow.
            */
           if (META_STACK_ID_IS_X11 (windows[i]) &&
-              meta_ui_window_is_dummy (display->screen->ui, windows[i]))
+              meta_ui_window_is_dummy (display->x11_display->ui, windows[i]))
             continue;
 
           if (stack_string->len > 0)
@@ -233,7 +234,7 @@ test_case_assert_stacking (TestCase *test,
           else
             g_string_append_printf (stack_string, "(%s)", window->title);
         }
-      else if (windows[i] == display->screen->guard_window)
+      else if (windows[i] == display->x11_display->guard_window)
         {
           if (stack_string->len > 0)
             g_string_append_c (stack_string, ' ');
@@ -284,7 +285,7 @@ test_case_check_xserver_stacking (TestCase *test,
 
   guint64 *windows;
   int n_windows;
-  meta_stack_tracker_get_stack (display->screen->stack_tracker, &windows, &n_windows);
+  meta_stack_tracker_get_stack (display->stack_tracker, &windows, &n_windows);
 
   for (i = 0; i < n_windows; i++)
     {
@@ -301,8 +302,8 @@ test_case_check_xserver_stacking (TestCase *test,
   Window parent;
   Window *children;
   unsigned int n_children;
-  XQueryTree (display->xdisplay,
-              meta_screen_get_xroot (display->screen),
+  XQueryTree (display->x11_display->xdisplay,
+              display->x11_display->xroot,
               &root, &parent, &children, &n_children);
 
   for (i = 0; i < (int)n_children; i++)
@@ -500,7 +501,8 @@ test_case_destroy (TestCase *test,
 
   async_waiter_destroy (test->waiter);
 
-  meta_display_set_alarm_filter (meta_get_display (), NULL, NULL);
+  meta_x11_display_set_alarm_filter (meta_get_display ()->x11_display,
+                                     NULL, NULL);
 
   g_hash_table_destroy (test->clients);
   g_free (test);
