@@ -50,11 +50,12 @@ typedef struct
   char *lock_file;
   int abstract_fd;
   int unix_fd;
-  pid_t pid;
   struct wl_client *client;
   struct wl_resource *xserver_resource;
   char *display_name;
 
+  GCancellable *xserver_died_cancellable;
+  GSubprocess *proc;
   GMainLoop *init_loop;
 
   MetaXWaylandSelection *selection_data;
@@ -71,6 +72,8 @@ struct _MetaWaylandCompositor
 
   MetaWaylandSeat *seat;
   MetaWaylandTabletManager *tablet_manager;
+
+  GHashTable *scheduled_surface_associations;
 };
 
 #endif /* META_WAYLAND_PRIVATE_H */
